from django import forms
from django.utils.functional import cached_property
from wagtail.blocks import FieldBlock

from wagtail_color_panel.validators import hex_triplet_validator
from wagtail_color_panel.widgets import ColorInputWidget


class NativeColorBlock(FieldBlock):
    def __init__(self, required=True, help_text=None, validators=(), **kwargs):
        self.field_options = {
            "required": required,
            "help_text": help_text,
            "max_length": 7,
            "min_length": 7,
            "validators": [hex_triplet_validator],
        }
        super().__init__(**kwargs)

    @cached_property
    def field(self):
        field_kwargs = {"widget": ColorInputWidget()}
        field_kwargs.update(self.field_options)
        return forms.CharField(**field_kwargs)

    class Meta:
        icon = "radio-full"



from django.db import models
from django.utils.translation import gettext_lazy as _

from wagtail_color_panel.validators import hex_triplet_validator


class ColorField(models.CharField):
    default_validators = [hex_triplet_validator]
    description = _("Hex triplet colors")

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("max_length", 7)
        super().__init__(*args, **kwargs)



import warnings

from wagtail.admin.panels import FieldPanel

from wagtail_color_panel.widgets import ColorInputWidget, PolyfillColorInputWidget


class NativeColorPanel(FieldPanel):
    def get_form_options(self):
        opts = super().get_form_options()
        opts["widgets"] = {
            self.field_name: ColorInputWidget(),
        }
        return opts


class PolyfillColorPanel(FieldPanel):
    """
    DEPRECATED: This panel uses the Spectrum library for IE11 compatibility.
    Since IE11 is no longer supported by Microsoft (as of June 2022) and this
    package requires Wagtail 6.3+ (which dropped IE11 support), please use
    NativeColorPanel instead.

    This panel will be removed in version 2.0.0.
    """

    def __init__(self, *args, **kwargs):
        warnings.warn(
            "PolyfillColorPanel is deprecated and will be removed in "
            "wagtail-color-panel 2.0.0. Use NativeColorPanel instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        super().__init__(*args, **kwargs)

    def get_form_options(self):
        opts = super().get_form_options()
        opts["widgets"] = {
            self.field_name: PolyfillColorInputWidget(),
        }
        return opts



import json

from django.forms import widgets
from django.utils.safestring import mark_safe
from wagtail import VERSION as WAGTAIL_VERSION

if WAGTAIL_VERSION >= (7, 1):
    from wagtail.admin.telepath import register
    from wagtail.admin.telepath.widgets import WidgetAdapter
else:
    from wagtail.telepath import register
    from wagtail.widget_adapters import WidgetAdapter


class PolyfillColorInputWidget(widgets.TextInput):
    class Media:
        css = {
            "all": (
                "https://cdnjs.cloudflare.com/ajax/libs/spectrum/1.8.0/spectrum.min.css",
            )
        }

        js = ("https://cdnjs.cloudflare.com/ajax/libs/spectrum/1.8.0/spectrum.min.js",)

    def render(self, name, value, attrs=None, renderer=None):
        out = super().render(name, value, attrs, renderer=renderer)
        field_id = attrs["id"]

        return mark_safe(
            out
            + """
            <script>
            (function(){
                function init() {
                    $("#__FIELD_ID__").spectrum({
                        showPalette: false,
                        preferredFormat: "hex",
                        showInput: true,
                    });
                }

                if (document.readyState === 'complete') {
                    init({});
                }

                $(window).on('load', function() {
                    init();
                });
            })();
            </script>
            """.replace(
                "__FIELD_ID__", field_id
            )
        )


if WAGTAIL_VERSION >= (6, 0):  # type: ignore
    from django.forms import Media

    class ColorInputWidget(widgets.TextInput):  # type: ignore
        template_name = "wagtail_color_panel/widgets/color-input-widget.html"

        def __init__(self, attrs=None):
            default_attrs = {
                "class": "color-input-widget__text-input",
            }
            attrs = attrs or {}
            attrs = {**default_attrs, **attrs}
            super().__init__(attrs=attrs)

        def build_attrs(self, *args, **kwargs):
            attrs = super().build_attrs(*args, **kwargs)
            attrs["data-controller"] = "color-input"
            return attrs

        @property
        def media(self):
            return Media(
                css={
                    "all": [
                        "wagtail_color_panel/css/color-input-widget.css",
                    ]
                },
                js=[
                    "wagtail_color_panel/js/color-input-widget.js",
                    "wagtail_color_panel/js/color-input-controller.js",
                ],
            )

else:
    from wagtail.utils.widgets import WidgetWithScript

    class ColorInputWidget(WidgetWithScript, widgets.TextInput):
        template_name = "wagtail_color_panel/widgets/color-input-widget.html"

        def __init__(self, attrs=None):
            default_attrs = {
                "class": "color-input-widget__text-input",
            }
            attrs = attrs or {}
            attrs = {**default_attrs, **attrs}
            super().__init__(attrs=attrs)

        def render_js_init(self, id_, name, value):
            return "new ColorInputWidget({0});".format(json.dumps(id_))

        class Media:
            css = {
                "all": [
                    "wagtail_color_panel/css/color-input-widget.css",
                ]
            }
            js = [
                "wagtail_color_panel/js/color-input-widget.js",
            ]


class ColorInputWidgetAdapter(WidgetAdapter):
    js_constructor = "wagtail_color_panel.widgets.ColorInput"

    class Media:
        js = [
            "wagtail_color_panel/js/color-input-widget-telepath.js",
        ]


register(ColorInputWidgetAdapter(), ColorInputWidget)



from django.core.exceptions import ValidationError
from django.test import TestCase
from wagtail.test.utils import WagtailTestUtils

from tests.testapp.factories import (
    PageWithColorFieldPageFactory,
    PageWithDefaultValuePageFactory,
)


class FieldTest(TestCase, WagtailTestUtils):
    def test_strings_larger_than_7_raises_error(self):
        with self.assertRaises(ValidationError) as _ctx:
            PageWithColorFieldPageFactory.create(color="#000000E")

    def test_value_not_beginning_with_hash_raises_error(self):
        with self.assertRaises(ValidationError) as ctx:
            PageWithColorFieldPageFactory.create(color="_000000")
        self.assertTrue("Enter a valid color hash" in str(ctx.exception))

    def test_invalid_color_hash_value_raises_error(self):
        with self.assertRaises(ValidationError) as ctx:
            PageWithColorFieldPageFactory.create(color="#0G0000")
        self.assertTrue("Enter a valid color hash" in str(ctx.exception))

    def test_valid_value_are_saved_and_treated_as_a_string(self):
        page = PageWithColorFieldPageFactory.create(color="#FF0000")
        self.assertEqual(page.color, "#FF0000")

    def test_field_gets_default_value(self):
        page = PageWithDefaultValuePageFactory.create()
        self.assertEqual(page.color, "#FF0000")



import re

from django.core.validators import RegexValidator
from django.utils.translation import gettext_lazy as _

hex_triplet_validator = RegexValidator(
    re.compile("^#[A-Fa-f0-9]{6}$"),
    message=_("Enter a valid color hash."),
    code="invalid",
)



from django.core.exceptions import ValidationError
from django.test import TestCase
from wagtail import VERSION as WAGTAIL_VERSION
from wagtail.test.utils import WagtailTestUtils

from wagtail_color_panel.blocks import NativeColorBlock
from wagtail_color_panel.widgets import ColorInputWidget


class BlockTest(TestCase, WagtailTestUtils):
    def test_color_block_render(self):
        block = NativeColorBlock()
        html = block.render("#333333")

        self.assertEqual("#333333", html)

    def test_form_uses_proper_input_type(self):
        block = NativeColorBlock()

        if WAGTAIL_VERSION >= (6, 0):
            html = block.field.widget.render(name="cat", value="#333333", attrs={})
        else:
            html = block.field.widget.render_html(name="cat", value="#333333", attrs={})

        self.assertEqual(block.field.widget.__class__, ColorInputWidget)
        self.assertIn('type="color"', html)

    def test_that_hex_colors_are_validated(self):
        block = NativeColorBlock()
        with self.assertRaises(ValidationError) as ctx:
            block.clean("Invalid")

        self.assertTrue("Enter a valid color hash" in str(ctx.exception))

    def test_color_block_supports_default_value(self):
        default_value = NativeColorBlock(default="#333333").get_default()
        self.assertEqual("#333333", default_value)



from django.test import TestCase
from wagtail.test.utils import WagtailTestUtils

from tests.testapp.factories import PageWithColorFieldPageFactory
from wagtail_color_panel.edit_handlers import NativeColorPanel
from wagtail_color_panel.widgets import ColorInputWidget

# from wagtail.admin.panels import get_form_for_model
# from wagtail.admin.forms import WagtailAdminModelForm, WagtailAdminPageForm


class PanelTest(TestCase, WagtailTestUtils):
    def test_field_to_panel_mapping(self):
        """
        PageWithColorFieldPageForm = get_form_for_model(
            PageWithColorField,
            form_class=WagtailAdminPageForm
        )
        form = PageWithColorFieldPageForm()
        """

    def test_native_color_panel_uses_correct_widget(self):
        page = PageWithColorFieldPageFactory.create(color="#000000")

        color_panel = page.content_panels[1]
        self.assertEqual(color_panel.__class__, NativeColorPanel)

        # Bind the panel to the model before calling get_form_options
        bound_panel = color_panel.bind_to_model(page.__class__)
        color_widget = bound_panel.get_form_options()["widgets"]["color"]
        self.assertEqual(color_widget.__class__, ColorInputWidget)



import io
import re
from pathlib import Path

from setuptools import find_packages, setup

install_requires = ["wagtail>=6.3"]

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

version = ""
with io.open("wagtail_color_panel/__init__.py", "r", encoding="utf8") as fd:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE
    ).group(1)

setup(
    name="wagtail-color-panel",
    version=version,
    description="Introduces panels for selecting colors in Wagtail.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Martin Sandström",
    author_email="martin@marteinn.se",
    url="https://github.com/marteinn/wagtail-color-panel",
    project_urls={
        "Bug Reports": "https://github.com/marteinn/wagtail-color-panel/issues",
        "Source": "https://github.com/marteinn/wagtail-color-panel",
        "Changelog": "https://github.com/marteinn/wagtail-color-panel/blob/develop/CHANGELOG.md",
    },
    packages=find_packages(
        exclude=("*.tests", "*.tests.*", "tests.*", "tests", "example*")
    ),
    include_package_data=True,
    install_requires=install_requires,
    zip_safe=False,
    extras_require={"test": ["pytest-django", "wagtail-factories", "pytest"]},
    license="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 4.2",
        "Framework :: Django :: 5.1",
        "Framework :: Django :: 5.2",
        "Framework :: Wagtail",
        "Framework :: Wagtail :: 6",
        "Framework :: Wagtail :: 7",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Topic :: Utilities",
    ],
    python_requires=">=3.9",
)
